// /**
//  * WebSocket事件处理函数接口
//  */
export const TioWs = (userId: string, bsId: string, groupId: string, handler: IHandler): WebSocketObject => {
    const wsObj: WebSocketObject = {
        ip: '192.168.50.137',
        port: 6888,
        url: '',
        reconnUrl: '',
        binaryType: 'blob',
        heartbeatTimeout: 18000,
        reconnInterval: 1000,
        heartbeatSendInterval: 9000, // 心跳发送间隔为超时时间的一半
        maxReconnAttempts: 10, // 增加重连次数为 10 次
        reconnAttempts: 0, // 当前重连尝试次数
        messageQueue: [], // 消息队列
        isProcessing: false, // 是否正在处理消息
        lastInteractionTimeValue: 0,
        ws: null,
        pingIntervalId: null,
        memoryCheckIntervalId: null,
        handler: handler,
        connect(isReconnect: boolean): WebSocket {
            const _url = isReconnect ? wsObj.reconnUrl : wsObj.url;
            console.log('正在尝试连接到:', _url);
            const ws = new WebSocket(_url);
            wsObj.ws = ws;

            ws.binaryType = wsObj.binaryType;

            // 连接成功时的处理
            ws.onopen = function (event: Event) {
                wsObj.reconnInterval = 1000; // 重置重连间隔
                wsObj.reconnAttempts = 0; // 重置重连尝试次数
                if (typeof wsObj.handler.onopen === 'function') {
                    wsObj.handler.onopen(event, ws);
                }
                wsObj.lastInteractionTime(new Date().getTime());
                wsObj.pingIntervalId = setInterval(() => wsObj.ping(), wsObj.heartbeatSendInterval);
                console.log('WebSocket 连接成功');
            };

            // 接收消息时的处理
            ws.onmessage = function (event: MessageEvent) {
                wsObj.messageQueue.push(event);
                if (!wsObj.isProcessing) {
                    wsObj.processQueue();
                }
            };

            // 连接关闭时的处理
            ws.onclose = function (event: CloseEvent) {
                clearInterval(wsObj.pingIntervalId);
                if (typeof wsObj.handler.onclose === 'function') {
                    wsObj.handler.onclose(event, ws);
                }
                wsObj.reconn(event);
                console.log('WebSocket 连接关闭:', event.code);
            };

            // 发生错误时的处理
            ws.onerror = function (event: Event | ErrorEvent) {
                console.error('WebSocket 错误:', event);
                if (typeof wsObj.handler.onerror === 'function') {
                    wsObj.handler.onerror(event, ws);
                }
                console.log('WebSocket 错误处理完成');
            };

            return ws;
        },

        // 重连函数
        reconn(event: CloseEvent): void {
            if (wsObj.reconnAttempts < wsObj.maxReconnAttempts) {
                console.log(`正在尝试重新连接 (${wsObj.reconnAttempts + 1}/${wsObj.maxReconnAttempts}):`, event);
                setTimeout(function () {
                    const ws = wsObj.connect(true);
                    wsObj.ws = ws;
                    wsObj.reconnAttempts++;
                }, wsObj.reconnInterval);
            } else {
                console.log('达到最大重连次数，停止重连');
            }
        },

        // 记录最后交互时间的函数
        lastInteractionTime(time?: number): number {
            if (time !== undefined) {
                wsObj.lastInteractionTimeValue = time;
            }
            return wsObj.lastInteractionTimeValue;
        },

        // 处理消息队列
        processQueue(): void {
            if (wsObj.messageQueue.length > 0) {
                wsObj.isProcessing = true;
                const event = wsObj.messageQueue.shift();
                if (typeof wsObj.handler.onmessage === 'function' && event) {
                    wsObj.handler.onmessage(event, wsObj.ws!);
                }
                wsObj.lastInteractionTime(new Date().getTime());
                setTimeout(wsObj.processQueue, 0);
            } else {
                wsObj.isProcessing = false;
            }
        },

        // 心跳检测函数
        ping(): void {
            const iv = new Date().getTime() - wsObj.lastInteractionTime();
            if (wsObj.heartbeatSendInterval + iv >= wsObj.heartbeatTimeout) {
                if (typeof wsObj.handler.ping === 'function') {
                    wsObj.handler.ping(wsObj.ws!);
                }
            }
        },

        // 发送消息函数
        send(data: any): void {
            console.log('正在发送消息:', data);
            if (wsObj.ws && wsObj.ws.readyState === WebSocket.OPEN) {
                wsObj.ws.send(data);
            } else {
                console.error('WebSocket未打开,无法发送消息。');
            }
        },

        // 关闭连接函数
        close(): void {
            console.log('关闭WebSocket连接');
            wsObj.reconnInterval = 999999; // 防止重连
            if (wsObj.ws) {
                wsObj.ws.close();
            }
            clearInterval(wsObj.pingIntervalId);
            clearInterval(wsObj.memoryCheckIntervalId);
            wsObj.messageQueue = []; // 清空消息队列
            wsObj.isProcessing = false;
        },

        // 清理不必要的数据
        clearUnnecessaryData(): void {
            console.log('正在清理不必要的数据...');
            // 清理可能存在的缓存
            if (wsObj.cache) {
                wsObj.cache = {};
            }
            // 如果有存储历史消息，可以只保留最近的一部分
            if (wsObj.messageHistory && wsObj.messageHistory.length > 1000) {
                wsObj.messageHistory = wsObj.messageHistory.slice(-1000);
            }
            // 重置一些可能占用内存的临时变量
            wsObj.tempData = null;
            console.log('清理完成');
        },

        // 检查内存使用情况
        checkMemoryUsage(): void {
            if (typeof performance !== 'undefined' && performance.memory) {
                const memoryUsage = performance.memory.usedJSHeapSize / (1024 * 1024);
                console.log(`内存使用: ${Math.round(memoryUsage * 100) / 100} MB`);
                if (memoryUsage > 100) {
                    console.warn('内存使用过高，正在尝试清理...');
                    // 清理消息队列
                    if (wsObj.messageQueue.length > 100) {
                        wsObj.messageQueue = wsObj.messageQueue.slice(-100);
                    }
                    // 清理不必要的数据
                    wsObj.clearUnnecessaryData();
                    // 如果清理后仍然超过阈值，考虑重启连接
                    setTimeout(() => {
                        const newMemoryUsage = performance.memory.usedJSHeapSize / (1024 * 1024);
                        if (newMemoryUsage > 500) {
                            console.warn('清理后内存仍然过高，正在重启连接...');
                            wsObj.restartConnection();
                        }
                    }, 1000);
                }
            }
        },

        restartConnection(): void {
            this.close();
            this.connect(false);
        },

        cache: null as any,
        messageHistory: null as unknown as any[],
        tempData: null as any
    };

    // 构建WebSocket的URL
    const paramStr = `userId=${userId}&bsId=${bsId}&groupId=${groupId}`;
    wsObj.url = `ws://${wsObj.ip}:${wsObj.port}/websocket/push${paramStr ? '?' + paramStr : ''}`;
    wsObj.reconnUrl = wsObj.url + '&tiows_reconnect=true';

    // WebSocket首次连接
    wsObj.connect(false);

    // 定期检查内存使用
    wsObj.memoryCheckIntervalId = setInterval(wsObj.checkMemoryUsage, 60000); // 每分钟检查一次

    return wsObj;
};
